#!/bin/bash

###########################################################
## Copyright (c) 2002-2024 Alexey Kuryakin daqgroup@mail.ru
###########################################################

###########################################################
## Execute /opt/daqgroup/share/lcard/linux/ldev-cpl.
## That is program to load/remove LCARD kernel drivers.
## ldev-cpl start   - start/load  LCARD kernel drivers.
## ldev-cpl stop    - stop/remove LCARD kernel drivers.
## ldev-cpl status  - query for   LCARD kernel drivers.
###########################################################

###########################################################
source $(crwkit which crwlib_base.sh); # Use base library #
source $(crwkit which crwlib_file.sh); # Use file library #
###########################################################

let fatal_notify_uses=0;

readonly packagename="daqgroup-lcard";         # name of target package
readonly optlcard="/opt/daqgroup/share/lcard"; # install target
readonly exe="$optlcard/linux/ldev-cpl";       # executable

###################################
# check deb package $1 is installed
###################################
function deb_found(){
 local status="$(dpkg -s $1 2>/dev/null | grep -iP "Status:\s*install\s*ok\s*installed")";
 if [ -n "$status" ]; then return 0; else return 1; fi;
};

######
# MAIN
######

function main(){
 if deb_found $packagename; then
  if [ -x "$exe" ]; then
   exec $exe "$@";
  else
   fatal 1 "Error: missed $exe\nPlease install $packagename package.";
  fi;
 else
  fatal 1 "Error: package $packagename is not installed.";
 fi;
};

main "$@";

##############
## END OF FILE
##############
